@@ -1,15 +1,18 @@
 import re
-from datetime import datetime, timedelta
-import redis
+from datetime import datetime
+from datetime import timedelta
 
+import redis
+from django.conf import settings
+from django.core.exceptions import ValidationError
+from django.core.validators import MaxValueValidator
+from django.core.validators import MinValueValidator
+from django.db import models
 from django.utils.datastructures import SortedDict
 from django.utils.translation import ugettext_lazy as _
-from django.db import models
-from django.core.validators import MaxValueValidator, MinValueValidator
-from django.core.exceptions import ValidationError
-from django.conf import settings
 
 from .utils import cached_property
+from .utils import PY3
 
 REDISBOARD_DETAIL_FILTERS = [re.compile(name) for name in getattr(settings, 'REDISBOARD_DETAIL_FILTERS', (
     'aof_enabled', 'bgrewriteaof_in_progress', 'bgsave_in_progress',
@@ -24,6 +27,9 @@ REDISBOARD_DETAIL_SECONDS_KEYS = getattr(settings, 'REDISBOARD_DETAIL_SECONDS_KE
     'uptime_in_seconds',
 ))
 
+REDISBOARD_SLOWLOG_LEN = getattr(settings, 'REDISBOARD_SLOWLOG_LEN', 10)
+
+
 def prettify(key, value):
     if key in REDISBOARD_DETAIL_SECONDS_KEYS:
         return key, timedelta(seconds=value)
@@ -32,6 +38,7 @@ def prettify(key, value):
     else:
         return key, value
 
+
 class RedisServer(models.Model):
     class Meta:
         unique_together = ('hostname', 'port')
@@ -43,15 +50,15 @@ class RedisServer(models.Model):
 
     label = models.CharField(
         _('Label'),
-        max_length = 50,
-        blank = True,
-        null = True,
+        max_length=50,
+        blank=True,
+        null=True,
     )
 
     hostname = models.CharField(
         _("Hostname"),
-        max_length = 250,
-        help_text = _('This can also be the absolute path to a redis socket')
+        max_length=250,
+        help_text=_('This can also be the absolute path to a redis socket')
     )
 
     port = models.IntegerField(_("Port"), validators=[
@@ -62,20 +69,19 @@ class RedisServer(models.Model):
 
     sampling_threshold = models.IntegerField(
         _("Sampling threshold"),
-        default = 1000,
-        help_text = _("Number of keys after which only a sample (of random keys) is shown on the inspect page.")
+        default=1000,
+        help_text=_("Number of keys after which only a sample (of random keys) is shown on the inspect page.")
     )
     sampling_size = models.IntegerField(
         _("Sampling size"),
-        default = 200,
-        help_text = _("Number of random keys shown when sampling is used. Note that each key translates to a RANDOMKEY call in redis.")
+        default=200,
+        help_text=_("Number of random keys shown when sampling is used. Note that each key translates to a RANDOMKEY call in redis.")
     )
 
     def clean(self):
         if not self.hostname.startswith('/') and not self.port:
             raise ValidationError(_('Please provide either a hostname AND a port or the path to a redis socket'))
 
-
     @cached_property
     def connection(self):
         if self.hostname.startswith('/'):
@@ -85,9 +91,9 @@ class RedisServer(models.Model):
             hostname = self.hostname
             unix_socket_path = None
         return redis.Redis(
-            host = hostname,
-            port = self.port,
-            password = self.password,
+            host=hostname,
+            port=self.port,
+            password=self.password,
             unix_socket_path=unix_socket_path,
         )
 
@@ -109,8 +115,9 @@ class RedisServer(models.Model):
                 'clients': info['connected_clients'],
                 'brief_details': SortedDict(
                     prettify(k, v)
-                    for k, v in sorted(info.items(), key=lambda (k,v): k)
-                    if any(name.match(k) for name in REDISBOARD_DETAIL_FILTERS)
+                    for name in REDISBOARD_DETAIL_FILTERS
+                    for k, v in (info.items() if PY3 else info.iteritems())
+                    if name.match(k)
                 )
             }
         except redis.exceptions.ConnectionError:
@@ -121,16 +128,15 @@ class RedisServer(models.Model):
                 'details': {},
                 'brief_details': {},
             }
-        except redis.exceptions.ResponseError, e:
+        except redis.exceptions.ResponseError as exc:
             return {
-                'status': 'ERROR: %s' % e.args,
+                'status': 'ERROR: %s' % exc.args,
                 'clients': 'n/a',
                 'memory': 'n/a',
                 'details': {},
                 'brief_details': {},
             }
 
-
     def __unicode__(self):
         if self.label:
             label = '%s (%%s)' % self.label
@@ -143,3 +149,23 @@ class RedisServer(models.Model):
             label = label % self.hostname
 
         return label
+
+    def slowlog_len(self):
+        try:
+            return self.connection.slowlog_len()
+        except redis.exceptions.ConnectionError:
+            return 0
+
+    def slowlog_get(self, limit=REDISBOARD_SLOWLOG_LEN):
+        try:
+            slowlog_get = self.connection.slowlog_get(REDISBOARD_SLOWLOG_LEN)
+            for i, (id, ts, duration, command) in enumerate(slowlog_get):
+                yield dict(
+                    id=id,
+                    ts=datetime.fromtimestamp(ts),
+                    duration=duration,
+                    command=command,
+                )
+
+        except redis.exceptions.ConnectionError:
+            pass
